<?php
namespace Models;

// Модель для работы с Фискальным регистратором
class Fr extends \DB\SQL\Mapper{
	public $db, $f3, $web, $sets, $method='POST', $messages, $ticket_type=array('1'=>0, '2'=>1);

	public function __construct($db, $f3) {
		parent::__construct($db, 'fr_settings' ); // подключаемся к таблице с настройками
		$this->f3=$f3;
		$this->db=$db;
		$this->web=\Web::instance();
		$this->messages=new \Models\Messages();
		$this->sets=$this->get_fr();
	}

	// Настройки ФР
	function get_fr() {
		$this->reset();
		$this->load();
		$sets=$this->cast();
		$sets['pays_type']=json_decode($sets['pays_type'], true);
		$sets['vat_type']=json_decode($sets['vat_type'], true);
		return $sets;
	}

	// Сохраняем настройки ФР
	function save_fr($post) {
		$post['pays_type']=json_encode($post['pays']);
		if(in_array(0, $post['vat']) && $post['active']==1) {
			return array(
				'success'=>false,
				'txt'=>'vat_not_null'
			);
		}
		$post['vat_type']=json_encode($post['vat']);
		unset($post['pays'], $post['vat']);
		$this->reset();
		$this->get_fr();
		$this->copyFrom($post);
		$this->update();
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
		return array(
			'success'=>true,
			'txt'=>'save_successful'
		);
	}

	// Тест сервера
	function test($driver, $restart=true) {
		// Тестовый запрос
		$url=$this->sets['url'];
		$options = array(
			'timeout'=>3,
    		'method'  => $this->method,
    		'content' => "data=test;\n"
		);
		$res=$this->web->request($url, $options);
		// Если в ответ не пришел код ответа 200, то надо запустить сервер
		if(mb_strpos($res['headers'][0], '200')===false) {
			// Если выбран драйвер принтера
			if($driver!='') {
				if($restart) {
					$script=$driver.'/server.py';
					include('drivers/start.php');
					$this->test($driver, false);
				}
				else {
					echo "Не удается запустить драйвер принтера";
					exit();
				}
			}
			else {
				echo "Сервер принтера не доступен";
				exit();
			}
		}
	}

	// Отправка запроса
	function send_ajax($data, $timeout=295, $view=false) {
		if(!$this->sets['active'] || $this->f3->get('hold')==1) return true;
		$this->test($this->sets['driver']);
		$url=$this->sets['url'];
		$options = array(
			'timeout'=>$timeout,
    		'method'  => $this->method,
    		'content' => 'data='.$data
		);
		$res=$this->web->request($url, $options);
		$continue_html= \Template::instance()->render('fr_error_continue.htm');
		// Проверяем ответ сревера
		if(mb_strpos($res['headers'][0], '200')===false) {
			if($res['error']) {
				echo $res['error'].$continue_html;
			}
			else {
				echo $res['body'].$continue_html;
			}
			exit();
		}
		// проверяем что смог сделать сервер
		$ok=true;
		$er='';
		$strs=explode("\n", $res['body']);
		$strs = array_diff($strs, array(''));
		$d=explode("\n", $data);
		$d = array_diff($d, array(''));
		foreach ($strs as $str) {
			$csv=str_getcsv($str, ';');
			if($csv[1]!=0) {
				$ok=false;
				$er.='<p class="false">Ошибка '.$csv[1].' '.$csv[2].'</p>';
			}
		}
		if($ok && count($strs)==count($d)) {
			if($view) {
				var_dump($res['body']);
				exit();
			}
			return true;
		}
		else {
			if($er=='') {
				echo '<pre>'.$data.'</pre>';
				echo '<pre>'.$res['body'].'</pre>';
				echo '<pre>Строк отправлено/получено:'.count($d).'/'.count($strs).'</pre>';
				$er='<p class="false">'.$this->messages->message('error')['txt'].'</p>';
			}
			echo $er;
			echo $continue_html;
			exit();
		}
	}

	// Открытие смены
	function open_session() {
		$strs="open_session;\n";
		if(!$this->sets['type']) {
			if($this->sets['header']) {
				$strs.=$this->sets['header']."\n";
			}
			$strs.="p;\n";
			$strs.="p;ОТКРЫТИЕ СМЕНЫ;\n";
			$strs.="p;".date("d-m-Y H:i").";\n";
			$strs.="p;\n";
			$strs.="p;Кассир ".$this->f3->get('SESSION.user.short_name').";\n";
			$strs.="p;\n";
		}

		if($this->sets['footer']) {
			$strs.=$this->sets['footer']."\n";
		}
		$this->send_ajax($strs);
	}

	// Закрытие смены
	function z($change) {
		$strs="z;\n";
		if(!$this->sets['type']) {
			$strs.=$this->report($change, 'z');
		}
		else {
			if($this->sets['footer']) {
				$strs.=$this->sets['footer']."\n";
			}
		}
		$this->send_ajax($strs);
	}

	// X-отчет смены
	function x($change) {
		$strs="x;\n";
		if(!$this->sets['type']) {
			$strs.=$this->report($change, 'x');
		}
		else {
			if($this->sets['footer']) {
				$strs.=$this->sets['footer']."\n";
			}
		}
		$this->send_ajax($strs);
	}

	function report($change, $type) {
		$types=array('x'=>'НУЛЕВОЙ ОТЧЕТ', 'z'=>'ОТЧЕТ С ГАШЕНИЕМ');
		$strs='';
		if($this->sets['header']) {
			$strs.=$this->sets['header']."\n";
		}
		$strs.="p;".$types[$type].";\n";
		$strs.="p;\n";

		if($change) {
			$receipts=new \Models\Receipts($this->db, $this->f3);
			$report=$receipts->change_statistic($change['id']);
			$p=new \Models\Paymentmethods($this->db, $this->f3);
			$paymentmethods=$p->select_method();
			foreach ($paymentmethods as $p) {
				$pays[$p['id']]=array('title'=>$p['title'], 'cash'=>$p['cash']);
			}
			$strs.="p;".date("d/m/Y H:i").";\n";
			$strs.="p;Смена ".$change['id'].";\n";
			$strs.="p;Начало смены ".date("d/m/Y H:i", $change['time_start']).";\n";
		}

		if($type=='z') {
			$strs.="p;Конец смены ".date("d/m/Y H:i").";\n";
		}
		$strs.="p;\n";

		if($change) {
			$types=array(
				$this->f3->get('receipt_sales')=>'Продажи',
				$this->f3->get('receipt_return')=>'Возвраты',
				$this->f3->get('receipt_cashin')=>'Внесение',
				$this->f3->get('receipt_cashout')=>'Изъятие'
			);
			foreach ($report as $type => $r) {
				$strs.="p;".$types[$type]." = ".$r['count'].";\n";
				foreach ($pays as $i => $p) {
					$strs.="p;  ".$p['title']." = ".number_format(abs($r['pay'][$i]), 2, '.', '').";\n";
					if($p['cash']==1) {
						if($type==$this->f3->get('receipt_return'))$total-=$r['pay'][$i];
						else $total+=$r['pay'][$i];
					}
				}
				$strs.="p;  Сумма = ".number_format(abs($r['sum']), 2, '.', '').";\n";
				$strs.="p;\n";
			}
			$strs.="p;Наличность в кассе = ".number_format($total, 2, '.', '')."\n";
			$strs.="p;\n";
		}

		if($this->sets['footer']) {
			$strs.=$this->sets['footer']."\n";
		}
		return $strs;
	}

	// Добавляем чек
	function ticket($t, $lines, $discount=0, $sales_receipts=0) {
		$user=$this->f3->get('SESSION.user.short_name');
		$vat_type=$this->f3->get('vat_type');
		$psr=1; // Признак способа расчета по умолчанию
		$ppr=1; // Признак предмета расчета по умолчанию
		$section=1; // Секция

		$strs='';
		// Заголовок
		if($this->sets['header']) {
			$strs.=$this->sets['header']."\n";
		}

		if($sales_receipts) {
			$strs.="p;Товарный чек;\n";
		}

		$strs.='b;'.($this->ticket_type[$t['type']]).';Кассир '.$user.";1;\n"; // Открываем чек

		if($t['client']) {
			$strs.="set_tlv;1008;5;".$t['client'].";\n";
		}

		// Товары
		foreach ($lines as $l) {
			if(!$l['discount'])$l['discount']=$discount;
			$title=mb_substr($l['product_title'], 0, 100);
			$price=$l['price']+($l['price']*($l['vat']/100))*max(0, $l['vat_type']);
			$price-=(float) $price*((float) $l['discount']/100);
			$price*=100;
			$price=round($price);
			$vt=$vat_type;
			if(!isset($l['ppr'])) $l['ppr']=$ppr;
			if($l['vat_type']==0) $vt=$this->sets['vat_type'][0];
			elseif($l['vat_type']!=0 && $l['vat']==0) $vt=$this->sets['vat_type'][1];
			elseif($l['vat_type']==1 && $l['vat']>0 && $l['vat']<=10) $vt=$this->sets['vat_type'][2];
			elseif($l['vat_type']==-1 && $l['vat']>0 && $l['vat']<=10) $vt=$this->sets['vat_type'][3];
			elseif($l['vat_type']==1 && $l['vat']>10) $vt=$this->sets['vat_type'][4];
			elseif($l['vat_type']==-1 && $l['vat']>10) $vt=$this->sets['vat_type'][5];
			$strs.='smde;'.$title.';'.$price.';'.round($l['count']*1000).';'.$vt.';'.$section.';';
			if($this->sets['version']=="1.1") {
				$strs.=$psr.';'.$l['ppr'].';';
			}
			$strs.="\n";
		}


		// оплата
		$strs.='tmde;';
		$pays=json_decode($t['payment'], true);
		// 1 способ оплаты, только для ФФД 1.05
		if($this->sets['version']=="1.05" && count($pays)==1) {
			$strs.=round($pays[0]['sum']*100).';'.max(0, $this->sets['pays_type'][$pays[0]['method']]).';';
		}
		// несколько способов оплаты или ФФД 1.1
		else {

			foreach ($pays as $p) {
				$mpay[$this->sets['pays_type'][$p['method']]]=$p['sum'];
			}

			if($this->sets['version']=='1.05') {
				$pay_count=4;
			}
			else {
				$pay_count=5;
			}

			for($i=0; $i<$pay_count; $i++) {
				$strs.=max(round($mpay[$i]*100), 0).';';
			}

			// Для ФФД 1.05 указываем налог для всех видов оплат
			if($this->sets['version']=="1.05") {
				for($i=0; $i<$pay_count; $i++) {
					//$strs.=$this->sets['sno'].';';
					$strs.="0;";
				}
			}
			// Для ФФД 1.1 указываем налог только 1 раз
			else {
				$strs.=$this->sets['sno'].';';
			}
		}

		$strs.="\n";
		$strs.="p;".date("d.m.Y H:i")." N ".$t['id'].";\n";
		if($this->sets['footer']) {
			$strs.=$this->sets['footer']."\n";
		}
		$this->send_ajax($strs);
	}

	// Внесение в кассу
	function cashin($change, $sum) {
		$strs="imde;".($sum*100).";\n";
		if(!$this->sets['type']) { 
			// Заголовок
			if($this->sets['header']) {
				$strs.=$this->sets['header']."\n";
			}
			if($sum>=0) {
				$strs.="p;ВНЕСЕНИЕ В КАССУ;\n";
			}
			else {
				$strs.="p;ИЗЪЯТИЕ ИЗ КАССЫ;\n";
			}
			$strs.="p;\n";
			$strs.="p;Сумма = ".number_format(abs($sum), 2, '.', '').";\n";
			$strs.="p;Кассир ".$this->f3->get('SESSION.user.short_name').";\n";
			$strs.="p;Смена ".$change.";\n";
			$strs.="p;".date("d-m-Y H:i").";\n";
			$strs.="p;\n";
		}

		// подвал
		if($this->sets['footer']) {
			$strs.=$this->sets['footer']."\n";
		}

		$this->send_ajax($strs);
	}

	// Обнуление не пробитого чека
	function reset_ticket() {
		$strs="g;";
		$this->send_ajax($strs);
		return array(
			'success'=>true,
			'txt'=>'ticket_reset'
		);
	}

	// Запрос полного статуса ККМ
	function status_kkm() {
		$strs="d;";
		$this->send_ajax($strs, null, true);
	}

	// Продолжить печать, после аварийной остановки
	function continue_print() {
		$strs="continue_print;";
		$this->send_ajax($strs);
		return array(
			'success'=>true,
			'txt'=>'print_continue'
		);
	}

	// Отчет по секциям
	function rep_sections() {
		$strs="rep_sections;";
		$this->send_ajax($strs);
	}

	// Печать копии последнего чека
	function c() {
		$strs='';
		// Заголовок
		if($this->sets['header']) {
			$strs.=$this->sets['header']."\n";
		}
		$strs.="c;\n";
		// Подвал
		if($this->sets['footer']) {
			$strs.=$this->sets['footer']."\n";
		}
		$this->send_ajax($strs);
	}

	// Печать пречека или товарного чека
	function print($receipt) {

		$strs='';
		// Заголовок
		if($this->sets['header']) {
			$strs.=$this->sets['header']."\n";
		}

		$strs.='pm;';

		// Товарный чек или пречек
		if($receipt['time']>0) {
			$strs.='#kkm_center#ТОВАРНЫЙ ЧЕК#kkm_br#';
		}
		else {
			$strs.='#kkm_center#ПРЕДВАРИТЕЛЬНЫЙ ЧЕК#kkm_br#';	
		}

		$sum=0;

		// Позиции в чеке
		foreach ($receipt['receiptlines'] as $l) {
			// Цена
			$price=$l['price']+($l['price']*($l['vat']/100))*max(0, $l['vat_type']);
			$price=round($price, 2);
			$price-=$price*($receipt['discount']/100);
			$sum+=$price*$l['count']; // Сумма чека
			$strs.=$l['product_title'].'#kkm_br#';
			$strs.='#kkm_right#'.number_format($l['count'], 3, '.', '').'X'.number_format($price, 2, '.', '').'='.number_format($price*$l['count'], 2, '.', '').'#kkm_br#';

			// НДС товара
			if($l['vat_type']!=0) {
				$vp=(($l['price']/(100+$l['vat']*abs(min(0, $l['vat_type']))))*$l['vat'])*$l['count'];
				$vp=round($vp, 2);
				$vat[$l['vat']]+=$vp; // Сумма НДС по чеку
				$strs.='НДС#kkm_right#'.$l['vat'].'%='.number_format($vp, 2, '.', '').'#kkm_br#';
			}
		}

		$strs.='#kkm_bold#ИТОГ#kkm_right#='.number_format($sum, 2, '.', '').'#kkm_br#';

		// Если чек пробит
		if($receipt['time']>0 && $receipt['pays']) {
			$cash=0;
			// Оплата
			$strs.='ОПЛАТА#kkm_br#';
			foreach ($receipt['pays'] as $p) {
				$cash+=$p['sum'];
				$strs.=$p['title'].'#kkm_right#='.number_format(round($p['sum'], 2), 2, '.', '').'#kkm_br#';
			}
			// Сдача
			$strs.='Сдача#kkm_right#='.number_format($cash-$sum, 2, '.', '').'#kkm_br#';
			$sno=array('01'=>'ОСН','02'=>'УСН','04'=>'УСН','08'=>'ЕНВД','10'=>'ЕСХН ','20'=>'Патент');
			// СНО
			$strs.='СНО#kkm_right#'.$sno[$this->sets['sno']].'#kkm_br#';
			// НДС по чеку
			if($vat) {
				$strs.='НДС#kkm_br#';
				foreach ($vat as $p=>$v) {
					$strs.='#kkm_right#'.number_format($p, 2, '.', '').'% = '.number_format($v, 2, '.', '').'#kkm_br#';
				}
			}
			// Кассир и смена
			$strs.='Кассир#kkm_br#';
			$strs.=$receipt['user']['short_name'].'#kkm_br#';
			$strs.='Смена '.$receipt['changes'].'#kkm_br#';
		}

		// Номер и дата чека
		$strs.='ЧЕК '.$receipt['id'].'#kkm_br#';
		if($receipt['time']>0) {
			$strs.=date('d-m-Y H:i', $receipt['time']);
		}
		$strs.="\n";

		// Подвал
		if($this->sets['footer']) {
			$strs.=$this->sets['footer']."\n";
		}
		$this->send_ajax($strs);
	}

	// Список драйверов
	function get_drivers() {
		$drivers=array();
		$printers_dir=$this->f3->get('printers_dir');
		$dirs=scandir($printers_dir); // Ищем все драйвера
		if($dirs) {
			foreach ($dirs as $d) {
				$folder=$printers_dir.$d;
				$file=$folder.'/server.config';
				if(is_dir($folder) && file_exists($file)) {
					$config=parse_ini_file($file);
					if($config) {
						$drivers[$folder]=array(
							'title'=>$config['ServerName'],
							'path'=>$folder
						);
					}
				}
			}
		}

		return $drivers;
	}
}
