<?php
namespace Models;

// Модель для настроек
class Settings extends Controller {
	public $db, $f3;

	public function __construct($f3, $db=null) {
		parent::__construct($f3);
		$this->db=$db;
		$this->f3=$f3;
	}

	// данные о страницах
	function page_data($page='api') {
		$pages=array(
			'api'=>array(
				'title'=>'Настройки подключения к API',
				'inc'=>'settings_api.htm'
			),
			'synchronization'=>array(
				'title'=>'Синхронизация с Агрота',
				'inc'=>'settings_synchronization.htm'
			),
			'kkm'=>array(
				'title'=>'Настройка Контрольно-кассовой машины',
				'inc'=>'settings_fr.htm'
			),
			'logs'=>array(
				'title'=>'Ошибки cинхронизации',
				'inc'=>'settings_logs.htm'
			),
			'cd'=>array(
				'title'=>'Настройка дисплея покупателя',
				'inc'=>'settings_cd.htm'
			),
			'service'=>array(
				'title'=>'Сервисное обслуживание',
				'inc'=>'settings_service.htm'
			),
			'base'=>array(
				'title'=>'Основные настройки',
				'inc'=>'settings_base.htm'
			),
			'rooms'=>array(
				'title'=>'Управление залами',
				'inc'=>'settings_rooms.htm'
			),
			'drivers'=>array(
				'title'=>'Настройка драйвера',
				'inc'=>'settings_drivers.htm'
			)
		);
		$pages[$page]['page']=$page;

		return $pages[$page];
	}

	// Получаем настройки из таблицы настроек
	function get_settings($name=array()) {
		$sets=array();
		if($name) {
			$sql="SELECT * FROM `settings` WHERE ";
			$where='';
			$val=array();
			foreach ($name as $n) {
				if($where!='')$where.=' OR ';
				$where.="`name`=?";
				$val[]=$n;
			}
			if($where!='') {
				$s=$this->db->exec($sql.$where, $val);
				if($s) {
					foreach ($s as $val) {
						$sets[$val['name']]=$val['value'];
					}
				}
			}
		}
		return $sets;
	}

	// Получаем основные настройки программы
	function get_base() {
		$name=array('auto_fullscreen', 'virtual_keyboard', 'descriptions_pattern', 'cleaning', 'auto_logout', 'cafe_mode');
		return $this->get_settings($name);
	}

	// Сохраняем основные настройки программы
	function save_base($post) {
		if(isset($post['auto_logout']) && !is_numeric($post['auto_logout']) || $post['auto_logout']<0) {
			return array(
				'success'=>false,
				'txt'=>'incorrect_auto_logout'
			);
		}
		$sql="INSERT OR REPLACE INTO `settings` (`name`, `value`) VALUES ";
		$val=array();
		foreach ($post as $key => $value) {
			if(!empty($val)) {
				$sql.=", ";
			}
			$sql.="(?, ?)";
			$val[]=$key;
			$val[]=$value;
		}
		$res=$this->db->exec($sql, $val);
		if($res!==false) {
			return array(
				'success'=>true,
				'txt'=>'save_successful'
			);
		}
		return array(
			'success'=>false,
			'txt'=>'db_error'
		);
	}
	
	// Сохраняем настройки API
	function post_api($post) {
		if(!$post['domen'] || !$post['pass'] || !$post['store_id'] || !$post['rm_id'] || !$post['count']) {
			return array(
				'success'=>false,
				'txt'=>'not_enough_data'
			);
		}
		$api=new \Models\Api($this->f3, $post['domen'], $post['pass'], $post['store_id'], $post['count']);
		
		$res=$api->users();
		if(!$res['success']) {
			$txt='';
			$m=new \Models\Messages();
			foreach ($res['error'] as $er) {
				$txt.=$m->message($er)['txt'].'<br>';
			}
			return array(
				'success'=>false,
				'txt'=>$txt,
				'debug'=>$res['debug']
			);
		}
		if(count($res['users'])<=0) {
			return array(
				'success'=>false,
				'txt'=>'cashbox_users_not_found'
			);
		}
		$txt="<?php\r\n";
		$txt.='$api_store_id="'.$post['store_id'].'";'."\n\r";
		$txt.='$api_rm_id="'.$post['rm_id'].'";'."\n\r";
		$txt.='$api_domen="'.$post['domen'].'";'."\n\r";
		$txt.='$api_pass="'.$post['pass'].'";'."\n\r";
		$txt.='$api_count="'.$post['count'].'";'."\n\r";
		$this->f3->write('app/api_config.php', $txt);
		$res=array(
			'success'=>true,
			'txt'=>'save_successful',
			'reroute'=>'/'
		);

		if($this->f3->get('SESSION.user')) {
			unset($res['reroute']);
		}
		return $res;
	}

	// Сохраняем настройки автосинхронизации
	function post_synchronization($db,$auto_sinch) {
		$res=$db->exec("UPDATE synchronization SET auto_sinch=?", $auto_sinch);
		if($res>=0) {
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'reload'=>'.auto_sinch_info',
				'close_all_window'=>true
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// сброс всех настроек и удаление базы данных
	function reset() {
		if(!@unlink('app/api_config.php')) {
			return array(
				'success'=>false,
				'txt'=>'file_is_not_delete'
			);
		}
		return array(
			'success'=>true,
			'txt'=>'save_successful',
			'reroute'=>'/'
		);
	}

	// Удаление старых чеков
	function cleaning() {
		// Узнаем чеки для удаления и формируем данные для запроса удаления строк
		$sql="SELECT id FROM `receipts` WHERE time<=? AND time>0";
		$time=time()-3600*24*30;
		$tickets=$this->db->exec($sql, $time);
		$in='';
		$val=array();
		if($tickets) {
			foreach ($tickets as $t) {
				if($in!='')$in.=', ';
				$in.='?';
				$val[]=$t['id'];
			}
		}
		if($in!='') {
			// Удаляем чеки
			$sql="DELETE FROM `receipts` WHERE `id` IN (".$in.")";
			if($this->db->exec($sql, $val)!==false) {
				$sql="DELETE FROM `receiptlines` WHERE `receipt_id` IN (".$in.")";
				$this->db->exec($sql, $val);
			}
		}
		$this->save_base(array('cleaning'=>time()));
		$this->db->exec("VACUUM");
	}

	// Сохраненеи настроек драйвера
	function save_driver($data) {
		$file=$data['path'].'/server.config';
		if(!file_exists($file)) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}
		if(file_put_contents($file, $data['txt'])) {
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'close_all_window'=>true
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'error'
			);
		}
	}

	// Запуск драйвера
	function start_driver($driver) {
		$script=$driver.'/server.py';
		if(!file_exists($script)) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}
		include('drivers/start.php');

		// Проверяем запуск драйвера
		$models=array(
			'printers'=>'\Models\Fr',
			'displays'=>'\Models\Cd'
		);
		$folders=explode('/', $driver);
		$m=new $models[$folders[1]]($this->db, $this->f3);
		//$m->test($driver);

		return array(
			'success'=>true,
			'txt'=>'success'
		);
	}
}
